'use strict';

var vue = require('vue');
var stream = require('@tanbo/stream');
var core = require('@textbus/core');
var platformBrowser = require('@textbus/platform-browser');

const adapterError = core.makeError('VueAdapter');
/**
 * Textbus 桥接 Vue 渲染能力适配器，用于在 Vue 项目中渲染 Textbus 数据
 */
class Adapter extends platformBrowser.DomAdapter {
    constructor(components, mount) {
        super(mount);
        this.onViewUpdated = new stream.Subject();
        this.components = {};
        this.componentRefs = new WeakMap();
        this.componentRendingStack = [];
        Object.keys(components).forEach(key => {
            const vueComponent = components[key];
            const setup = vueComponent.setup;
            const self = this;
            vueComponent.setup = function (props) {
                const component = props.component;
                const vueInstance = vue.getCurrentInstance();
                const sub = component.changeMarker.onChange.subscribe(() => {
                    if (component.changeMarker.dirty) {
                        vueInstance.proxy.$forceUpdate();
                    }
                });
                vue.onMounted(() => {
                    if (props.rootRef.value) {
                        self.componentRootElementCaches.set(component, props.rootRef.value);
                    }
                    else {
                        self.componentRootElementCaches.remove(component);
                    }
                });
                vue.onUpdated(() => {
                    component.changeMarker.rendered();
                    self.onViewUpdated.next();
                });
                vue.onUnmounted(() => {
                    sub.unsubscribe();
                });
                return setup(props);
            };
            this.components[key] = vueComponent;
        });
    }
    componentRender(component) {
        const comp = this.components[component.name] || this.components['*'];
        if (comp) {
            let rootRef = this.componentRefs.get(component);
            if (!rootRef) {
                rootRef = vue.ref();
                this.componentRefs.set(component, rootRef);
            }
            component.__slots__.length = 0;
            this.componentRendingStack.push(component);
            const vNode = vue.h(comp, {
                component,
                rootRef,
                key: component.id
            });
            this.componentRendingStack.pop();
            return vNode;
        }
        throw adapterError(`cannot found view component \`${component.name}\`!`);
    }
    slotRender(slot, slotHostRender, renderEnv) {
        const context = this.componentRendingStack[this.componentRendingStack.length - 1];
        context.__slots__.push(slot);
        const vElement = slot.toTree(slotHostRender, renderEnv);
        this.slotRootVElementCaches.set(slot, vElement);
        const vNodeToJSX = (vNode) => {
            const children = [];
            for (let i = 0; i < vNode.children.length; i++) {
                const child = vNode.children[i];
                if (child instanceof core.VElement) {
                    children.push(vNodeToJSX(child));
                }
                else if (child instanceof core.VTextNode) {
                    children.push(core.replaceEmpty(child.textContent));
                }
                else {
                    children.push(this.componentRender(child));
                }
            }
            const props = Object.assign({}, (Array.from(vNode.attrs).reduce((a, b) => {
                a[b[0]] = b[1];
                return a;
            }, {})));
            if (vNode.classes.size) {
                props.class = Array.from(vNode.classes).join(' ');
            }
            if (vNode.styles) {
                props.style = Array.from(vNode.styles).reduce((a, b) => {
                    a[b[0]] = b[1];
                    return a;
                }, {});
            }
            return vue.h(vNode.tagName, props, ...children);
        };
        const refFn = (nativeNode) => {
            if (!nativeNode) {
                this.slotRootNativeElementCaches.remove(slot);
            }
            else {
                this.slotRootNativeElementCaches.set(slot, nativeNode);
            }
        };
        const currentRef = vElement.attrs.get('ref');
        if (currentRef) {
            vElement.attrs.set('ref', (v) => {
                refFn(v);
                if (typeof currentRef === 'function') {
                    currentRef(v);
                }
                else if (!currentRef.value) {
                    currentRef.value = v;
                }
            });
        }
        else {
            vElement.attrs.set('ref', refFn);
        }
        slot.changeMarker.rendered();
        return vNodeToJSX(vElement);
    }
}

exports.Adapter = Adapter;
